
local clk = os.clock()

local common = loadfile(CM_DIR .. 'cm_common.lua')
hc.set_global('CM_COMMON', string.dump(common))
common()

local Parsers = {}

_CM_Options = {}

_CM_Options.SystemCoding = os.setlocale('', 'all'):match('.*%D(%d+)')	--       ,    ,  ,
_CM_Options.SystemCoding = _CM_Options.SystemCoding and tonumber(_CM_Options.SystemCoding)	--      (  -  )
_CM_Options.LanguageID = _CM_Options.LanguageID or hc.language_id	--      ,     
-- hc.put_to_log('_CM_Options.SystemCoding = ', _CM_Options.SystemCoding, '\r\n_CM_Options.LanguageID = ', _CM_Options.LanguageID)

--    
hc.set_global_table_item('AdMuncherFiles', 'december.js', _CM_ReadFile(CM_DIR .. 'december.js'))
hc.set_global_table_item('AdMuncherFiles', 'december.css', _CM_ReadFile(CM_DIR .. 'december.css'))

-- -------------------------    --------------------------------
local lng = PrepareLanguage(_CM_Options.LanguageID)

local prs = [[
	#Parser ContentMaster
		Remove by class/id
		Remove by comment
		Remove scripts with text
		Remove by size
		Replace text
		Insert CSS
		No filtering on URL
		Remove interactively
	#/Parser

	#Parser HandyCache
		Remove elements by URL
		Block retrieval of URL
		Use White list
	#/Parser

	#Parser AdBlockPlus
		Remove elements by URL
		Element hiding
	#/Parser

	#Parser AdMuncher
		Remove scripts with text
		Remove tables with text
		Remove divs/spans with text
		Remove forms with text
		Remove images with "alt" text
		Block retrieval of URL
		Remove links to URL
		Remove images/etc with URL
		No filtering on URL
		Add javascript to all pages
		Add CSS to all pages
		Replace text
		Don't match against keyword
	#/Parser
]]
local iParsers = {}
for parser,Groups in prs:gmatch('#Parser%s+(%S+)(.-)#/Parser') do
	table.insert(iParsers, { Name=parser })
	Parsers[parser] = Parsers[parser] or {}
	for filter in Groups:gmatch('[\r\n]+%s*([^-%s][^\r\n]*)') do
		table.insert(iParsers[#iParsers], filter)
		Parsers[parser][filter]=true
	end
end
hc.set_global('CM_iPARSERS', SaveTable(iParsers))


local _, Users = xpcall(function() return dofile(hc.script_name:match('.*%.') .. 'ini') end, function() return { ALL={ Name='ALL', Parsers={}, Log={}, BypassNon200Answers=true, BypassNoHits=true } } end)
Users = Users or { ALL={ Name='ALL', Parsers={}, Log={}, BypassNon200Answers=true, BypassNoHits=true } }

local tmp_users = hc.get_users_stat('#all') or {}
for user,v in pairs(tmp_users) do	--    ''   ,    ini,   
	if not Users[user] then
		Users[user] = { Parent=Users.ALL, Name=user, Inherited=true, Parsers={} }
		table.insert(Users.ALL, Users[user])
	end
end

for parser,v in pairs(Parsers) do	--        'ALL'
	Users.ALL.Parsers[parser] = Users.ALL.Parsers[parser] or { On=nil, Hits=0, Files={}, Filters={} }
	for filter in pairs(v) do
		Users.ALL.Parsers[parser].Filters[filter] = Users.ALL.Parsers[parser].Filters[filter] or { On=nil, Hits=0 }
	end
end

--          
local function Inherit(ParentUser)
	for _,ChildUser in ipairs(ParentUser) do	--     
		for m,ParentParser in pairs(ParentUser.Parsers) do
			local ChildParser = ChildUser.Parsers[m]
			if ChildParser then
				for _,ParentFile in ipairs(ParentParser.Files) do
					local finded
					for _,ChildFile in ipairs(ChildParser.Files) do
						if ChildFile.Name==ParentFile.Name then
							finded = true
							break
						end
					end
					if not finded then table.insert(ChildParser.Files, { Name=ParentFile.Name, On=ParentFile.On, Inherited=true }) end
				end
				for filter,v in pairs(ParentParser.Filters) do
					ChildParser.Filters[filter] = ChildParser.Filters[filter] or { On=v.On, Hits=0, Inherited=true }
				end
			else
				ChildUser.Parsers[m] = { On=ParentParser.On, Hits=0, Files=loadstring(SaveTable(ParentUser.Parsers[m].Files))(), Filters={}, Inherited=true }
				for filter,v in pairs(ParentParser.Filters) do
					ChildUser.Parsers[m].Filters[filter] = { On=v.On, Hits=0, Inherited=true }
				end
			end
		end
		Inherit(ChildUser)
	end
end
Inherit(Users.ALL)
Users.ALL.BypassModeHotkey = Users.ALL.BypassModeHotkey or ''
Users.ALL.InteractiveModeHotkey = Users.ALL.InteractiveModeHotkey or ''
Users.ALL.AutoUpdate = Users.ALL.AutoUpdate or {}
Users.ALL.Editor = Users.ALL.Editor or 'notepad.exe'
hc.set_global('CM_USERS', SaveTable(Users))

hc.set_global('CM_AutoUpdate', Users.ALL.AutoUpdate)

-- -------------------------   AdMuncher --------------------------------
hc.set_global_table_item('AdMuncherFiles', '-#jsmain#-', _CM_ReadFile(CM_DIR .. 'admuncher.js'):gsub([[http://interceptedby%.admuncher%.com/.-/([^'"]*)]], 'http://local.cm/%1'))
hc.set_global_table_item('AdMuncherFiles', 'toolbar_iframe', _CM_ReadFile(CM_DIR .. 'admuncher_toolbar_iframe.htm'))
hc.set_global_table_item('AdMuncherFiles', 'toolbar_transparent', _CM_ReadFile(CM_DIR .. 'admuncher_toolbar_transparent.gif'))
hc.set_global_table_item('AdMuncherFiles', 'toolbar_sprite', _CM_ReadFile(CM_DIR .. 'admuncher_toolbar_sprite.png'))

local f = io.open(CM_DIR .. 'source\\compile.lua', 'rb')
if f then
	f:close()
	dofile(CM_DIR .. 'source\\compile.lua')
end
hc.set_global_table_item('CM_WORKERS', 'URL', _CM_ReadFile(CM_DIR .. 'workers\\url.cm'))
hc.set_global_table_item('CM_WORKERS', 'HTML', _CM_ReadFile(CM_DIR .. 'workers\\html.cm'))
hc.set_global_table_item('CM_WORKERS', 'JS', _CM_ReadFile(CM_DIR .. 'workers\\js.cm'))
hc.set_global_table_item('CM_WORKERS', 'CSS', _CM_ReadFile(CM_DIR .. 'workers\\css.cm'))
hc.set_global_table_item('CM_PARSERS', 'HTML', loadstring(_CM_ReadFile(CM_DIR .. 'workers\\parser_html.cm'))())


-- ------------------------- Auto Update --------------------------------
-- local uperiod = options:match('AutoUpdatePeriod=(%S+)')
-- local ulast = options:match('LastUpdateTime=(%S+)')
-- hc.set_global('CM_AutoUpdate', hc.get_global('CM_AutoUpdate') or { period=tonumber(uperiod), last=tonumber(ulast) })

_CM_Options.LAST_OPTIONS_CHANGE_TIME = os.time()

_CM_Options.Parsers = Parsers

local f = io.open(CM_DIR .. 'Content_Master.lua', 'rb')
if f then
	_CM_Options.Version = f:read('*a'):match('@version%s*([^\r\n]*)')
	f:close()
end

hc.set_global('CM_OPTIONS', _CM_Options)

hc.set_global('CM_RULES')

hc.set_global('CM_HITS', {})

hc.put_to_log('Content Master: initialized from ', hc.script_name:match('.*%.'), 'ini')
-- hc.put_msg(3, 'Content Master:\r\n', lng['Initialization finished'], string.format('\r\n%.2f ', os.clock()-clk), lng['sec'])
